#pragma ident "$Id: datreq.c,v 1.13 2004/09/28 22:55:44 dechavez Exp $"
/*======================================================================
 *
 *  ISI/IACP data request support
 *
 *====================================================================*/
#include "isi.h"
#include "util.h"

/* Initialize a generic time series header */

VOID isiInitGenericTSHDR(ISI_GENERIC_TSHDR *src)
{
    memset(src, 0, sizeof(ISI_GENERIC_TSHDR));
}

/* Print the contents of a data request */

VOID isiPrintReqTime(FILE *fp, REAL64 value)
{
char sbuf[256];
static char *oldest_string = "<  beg of disk loop >";
static char *newest_string = "<  end of disk loop >";
static char *keepup_string = "<       forever     >";

    if (value == ISI_OLDEST) {
        fprintf(fp, " %s", oldest_string);
    } else if (value == ISI_NEWEST) {
        fprintf(fp, " %s", newest_string);
    } else if (value == ISI_KEEPUP) {
        fprintf(fp, " %s", keepup_string);
    } else {
        fprintf(fp, " %s", utilDttostr(value, 0, sbuf));
    }
}

VOID isiPrintStreamReq(FILE *fp, ISI_STREAM_REQUEST *sreq)
{
    fprintf(fp, "'%s'.'%s'.'%s'", sreq->name.sta, sreq->name.chn, sreq->name.loc);
    isiPrintReqTime(fp, sreq->beg);
    isiPrintReqTime(fp, sreq->end);
}

VOID isiPrintDatreq(FILE *fp, ISI_DATA_REQUEST *datreq)
{
int i;

    fprintf(fp, "*** beg ISI_DATA_REQUEST ***\n");
    fprintf(fp, "     policy = %lu (%s)\n", datreq->policy, isiPolicyString(datreq->policy));
    fprintf(fp, "     format = %lu (%s)\n", datreq->format, isiFormatString(datreq->format));
    fprintf(fp, "   compress = %lu (%s)\n", datreq->compress, isiCompressString(datreq->compress));
    fprintf(fp, "    nstream = %lu\n", datreq->nstream);
    for (i = 0; i < datreq->nstream; i++) {
        fprintf(fp, "   %d: ", i+1);
        isiPrintStreamReq(fp, &datreq->stream[i]);
        fprintf(fp, "\n");
    }
    fprintf(fp, "*** end ISI_DATA_REQUEST ***\n");
}

VOID isiSetDatreqPolicy(ISI_DATA_REQUEST *dreq, UINT32 value)
{
    if (dreq == NULL) return;
    dreq->policy = value;
}

VOID isiSetDatreqFormat(ISI_DATA_REQUEST *dreq, UINT32 value)
{
    if (dreq == NULL) return;
    dreq->format = value;
}

VOID isiSetDatreqCompress(ISI_DATA_REQUEST *dreq, UINT32 value)
{
    if (dreq == NULL) return;
    dreq->compress = value;
}

UINT32 isiGetDatreqCompress(ISI_DATA_REQUEST *dreq)
{
    if (dreq == NULL) {
        errno = EINVAL;
        return ISI_COMP_UNDEF;
    }

    return dreq->compress;
}

/* initialize a data request structure */

VOID isiInitDataRequest(ISI_DATA_REQUEST *req)
{
    req->policy   = ISI_DEFAULT_RECONNECT_POLICY;
    req->format   = ISI_DEFAULT_FORMAT;
    req->compress = ISI_DEFAULT_COMP;
    req->nstream  = 0;
    req->stream   = NULL;
}

/* clear a data request structure */

VOID isiClearDataRequest(ISI_DATA_REQUEST *req)
{
    if (req == NULL) return;

    if (req->nstream > 0 && req->stream != NULL) {
        free(req->stream);
        req->stream = NULL;
        req->nstream = 0;
    }
    isiInitDataRequest(req);
}

ISI_STREAM_REQUEST *isiLocateStreamRequest(ISI_STREAM_NAME *target, ISI_DATA_REQUEST *datreq)
{
UINT32 i;

    if (target == NULL || datreq == NULL) {
        errno = EINVAL;
        return NULL;
    }

    for (i = 0; i < datreq->nstream; i++) {
        if (isiStreamNameMatch(&datreq->stream[i].name, target)) return &datreq->stream[i];
    }

    return NULL;
}

/* convert a string of the form sta.chn.loc into an ISI_STREAM_NAME */

BOOL isiBuildStreamName(char *input, ISI_STREAM_NAME *out)
{
int ntoken = 0;
#define STA_CHN_LOC_TOKEN_COUNT 3
#define STA_CHN_TOKEN_COUNT    (STA_CHN_LOC_TOKEN_COUNT - 1)
char *copy, *token[STA_CHN_LOC_TOKEN_COUNT];
static char *BlankLoc = "  ";

    if (input == NULL || out == NULL) {
        errno = EINVAL;
        return FALSE;
    }

    if ((copy = strdup(input)) == NULL) return FALSE;
    ntoken = utilParse(copy, token, ISI_STA_CHN_LOC_DELIMITERS, STA_CHN_LOC_TOKEN_COUNT, 0);
    if (ntoken == STA_CHN_TOKEN_COUNT) {
        token[STA_CHN_TOKEN_COUNT] = BlankLoc;
    } else if (ntoken != STA_CHN_LOC_TOKEN_COUNT) {
        free(copy);
        return FALSE;
    }

    memcpy(out->sta, token[0], ISI_STALEN); out->sta[ISI_STALEN] = 0;
    memcpy(out->chn, token[1], ISI_CHNLEN); out->chn[ISI_CHNLEN] = 0;
    memcpy(out->loc, token[2], ISI_LOCLEN); out->loc[ISI_LOCLEN] = 0;

    free(copy);
    return TRUE;
}

LIST *isiBuildStreamRequestList(REAL64 beg, REAL64 end, char *StreamSpec)
{
LIST *list, *TokenList;
LIST_NODE *crnt;
ISI_STREAM_REQUEST sreq;
static char *DefaultSpec = ISI_DEFAULT_STREAMSPEC;
static char *fid = "isiBuildStreamRequestList";

    if (StreamSpec == NULL) StreamSpec = DefaultSpec;

    if ((list = listCreate()) == NULL) return NULL;
    if ((TokenList = utilStringTokenList(StreamSpec, "+", 0)) == NULL) return NULL;
    
    sreq.beg = beg;
    sreq.end = end;
    crnt = listFirstNode(TokenList);
    while (crnt != NULL) {
        if (!isiBuildStreamName((char *) crnt->payload, &sreq.name)) {
            listDestroy(TokenList);
            return NULL;
        }
        if (!listAppend(list, &sreq, sizeof(ISI_STREAM_REQUEST))) {
            listDestroy(TokenList);
            return NULL;
        }
        crnt = listNextNode(crnt);
    }

    listDestroy(TokenList);
    return list;
}

/* send a data request */

static BOOL SendStreamRequest(IACP *iacp, ISI_STREAM_REQUEST *sreq)
{
UINT8 *ptr, *pLen;
UINT32 len;
IACP_FRAME frame;
#define LOCAL_BUFLEN (sizeof(ISI_STREAM_REQUEST) * 2)
UINT8 buf[LOCAL_BUFLEN];

    memset(buf, 0xee, LOCAL_BUFLEN);

    frame.payload.type = ISI_IACP_REQ_STREAM;
    frame.payload.data = buf;
    frame.payload.len  = isiPackStreamRequest(frame.payload.data, sreq);

    return iacpSendFrame(iacp, &frame);
}

BOOL isiIacpSendDataRequest(IACP *iacp, ISI_DATA_REQUEST *datreq)
{
int i;

    if (!iacpSendUINT32(iacp, ISI_IACP_REQ_FORMAT, datreq->format)) return FALSE;
    if (!iacpSendUINT32(iacp, ISI_IACP_REQ_COMPRESS, datreq->compress)) return FALSE;
    if (!iacpSendUINT32(iacp, ISI_IACP_REQ_POLICY, datreq->policy)) return FALSE;
    for (i = 0; i < datreq->nstream; i++) {
        if (!SendStreamRequest(iacp, &datreq->stream[i])) return FALSE;
    }
#ifdef DEBUG
    printf("isiIacpSendDataRequest\n");
    isiPrintDatreq(stdout, datreq);
#endif
    return iacpSendUINT32(iacp, 0, IACP_TYPE_NULL);
}

/* Append a stream request to the incoming request packet */

UINT32 isiAppendStreamReq(ISI_INCOMING *incoming, IACP_FRAME *frame)
{
ISI_STREAM_REQUEST sreq;

    if (incoming == NULL || frame == NULL) {
        errno = EINVAL;
        return IACP_ALERT_OTHER_ERROR;
    }

/* Make sure peer isn't mixing request types */

    if (incoming->list.count == 0) {
        incoming->type = frame->payload.type;
    } else if (incoming->type != frame->payload.type) {
        return IACP_ALERT_PROTOCOL_ERROR;
    }

/* Decode the message and append it to the list */

    isiUnpackStreamRequest(frame->payload.data, &sreq);
    listAppend(&incoming->list, &sreq, sizeof(ISI_STREAM_REQUEST));

    return IACP_ALERT_NONE;
}

/* Copy a list of stream requests into a data request array */

BOOL isiCopyStreamListToDataRequest(LIST *list, ISI_DATA_REQUEST *output)
{
UINT32 i;
ISI_STREAM_REQUEST *entry;

    if (!listSetArrayView(list)) return FALSE;
    if ((output->nstream = list->count) == 0) return TRUE;

    output->stream = (ISI_STREAM_REQUEST *) malloc(list->count * sizeof(ISI_STREAM_REQUEST));
    if (output->stream == NULL) return FALSE;

    for (i = 0; i < output->nstream; i++) {
        entry = (ISI_STREAM_REQUEST *) list->array[i];
        memcpy(&output->stream[i].name, &entry->name, sizeof(ISI_STREAM_NAME));
        output->stream[i].beg = entry->beg;
        output->stream[i].end = entry->end;
    }
    return TRUE;
}

/* Client level data request (with acknowledgement) */

static UINT32 ReceiveDataRequestAcknowledgment(ISI *isi, UINT32 *error)
{
UINT32 i;
BOOL finished;
static char *fid = "isiSendDataRequest:ReceiveDataRequestAcknowledgment";

    isiResetIncoming(&isi->incoming);
    isiInitDataRequest(&isi->datreq);

/* Read response into linked list */

    finished = FALSE;
    while (!finished) {
        if (iacpRecvFrame(isi->iacp, &isi->frame, isi->buf, ISI_INTERNAL_BUFLEN)) {
            switch (isi->frame.payload.type) {
              case ISI_IACP_REQ_FORMAT:
                utilUnpackUINT32(isi->frame.payload.data, &isi->datreq.format);
                break;
              case ISI_IACP_REQ_COMPRESS:
                utilUnpackUINT32(isi->frame.payload.data, &isi->datreq.compress);
                break;
              case ISI_IACP_REQ_POLICY:
                utilUnpackUINT32(isi->frame.payload.data, &isi->datreq.policy);
                break;
              case ISI_IACP_REQ_STREAM:
                *error = isiAppendStreamReq(&isi->incoming, &isi->frame);
                if (*error != IACP_ALERT_NONE) {
                    isiLogMsg(isi, LOG_ERR, "%s: isiAppendStreamReq: %s", fid, strerror(errno));
                    return FALSE;
                }
                break;
              case IACP_TYPE_NULL:
                finished = TRUE;
                break;
              case IACP_TYPE_ALERT:
                finished = TRUE;
                *error = iacpAlertCauseCode(&isi->frame);
                break;
              case IACP_TYPE_NOP:
                break;
              default:
                isiLogMsg(isi, LOG_ERR, "%s: unexpected type %lu frame received", fid, isi->frame.payload.type);
                *error = IACP_ALERT_PROTOCOL_ERROR;
                return FALSE;
            }
        }
    }

/* Copy from linked list to internal data request buffer */

    if (!isiCopyStreamListToDataRequest(&isi->incoming.list, &isi->datreq)) {
        isiLogMsg(isi, LOG_ERR, "%s: isiCopyStreamListToDataRequest: %s", fid, strerror(errno));
        *error = IACP_ALERT_OTHER_ERROR;
        return FALSE;
    }

/* Initially all requested streams have incomplete status */

    for (i = 0; i < isi->datreq.nstream; i++) isi->datreq.stream[i].status = ISI_INCOMPLETE;

    return TRUE;
}

BOOL isiSendDataRequest(ISI *isi, ISI_DATA_REQUEST *datreq, UINT32 *error)
{
static char *fid = "isiSendDataRequest";

    if (error == NULL) {
        errno = EINVAL;
        return FALSE;
    }

    if (isi == NULL || datreq == NULL) {
        *error = IACP_ALERT_OTHER_ERROR;
        errno = EINVAL;
        return FALSE;
    }

    if (!isiIacpSendDataRequest(isi->iacp, datreq)) {
        isiLogMsg(isi, LOG_ERR, "%s: isiIacpSendDataRequest failed", fid);
        *error = IACP_ALERT_IO_ERROR;
        return FALSE;
    }

    return ReceiveDataRequestAcknowledgment(isi, error);
}

ISI_DATA_REQUEST *isiFreeDataRequest(ISI_DATA_REQUEST *dreq)
{
    if (dreq == NULL) return NULL;
    isiClearDataRequest(dreq);
    free(dreq);
    return NULL;
}

ISI_DATA_REQUEST *isiAllocSimpleDataRequest(REAL64 beg, REAL64 end, char *spec)
{
int i;
LIST *list;
ISI_DATA_REQUEST *dreq;
static char *fid = "isiAllocSimpleDataRequest";

    dreq = (ISI_DATA_REQUEST *) malloc(sizeof(ISI_DATA_REQUEST));
    if (dreq == NULL) return NULL;

    isiInitDataRequest(dreq);
    list = isiBuildStreamRequestList(beg, end, spec);
    if (list == NULL) return isiFreeDataRequest(dreq);

    if (!listSetArrayView(list)) return isiFreeDataRequest(dreq);

    dreq->nstream = list->count;
    dreq->stream = (ISI_STREAM_REQUEST *) malloc(dreq->nstream * sizeof(ISI_STREAM_REQUEST));
    if (dreq->stream == NULL) return isiFreeDataRequest(dreq);

    for (i = 0; i < list->count; i++) dreq->stream[i] = *((ISI_STREAM_REQUEST *) list->array[i]);

    listDestroy(list);
    return dreq;
}

ISI_DATA_REQUEST *isiAllocDefaultDataRequest(VOID)
{
    return isiAllocSimpleDataRequest(ISI_DEFAULT_BEGTIME, ISI_DEFAULT_ENDTIME, ISI_DEFAULT_STREAMSPEC);
}

/* Revision History
 *
 * $Log: datreq.c,v $
 * Revision 1.13  2004/09/28 22:55:44  dechavez
 * added conditional DEBUG support
 *
 * Revision 1.12  2004/06/21 19:47:43  dechavez
 * print name components in quotes in isiPrintStreamReq() (helps debugging)
 *
 * Revision 1.11  2004/06/10 17:35:34  dechavez
 * allow for sta.chn strings in isiBuildStreamName()
 *
 * Revision 1.10  2004/06/04 22:49:43  dechavez
 * various AAP windows portability modifications
 *
 * Revision 1.9  2004/01/29 18:40:25  dechavez
 * init SendStreamRequest() buf with 0xee's (debug aid)
 *
 * Revision 1.8  2003/12/11 22:19:23  dechavez
 * changed isiFreeDataRequest() to free streams via isiClearDataRequest()
 * instead of its own explicit free()
 *
 * Revision 1.7  2003/12/10 05:50:34  dechavez
 * added IACP_TYPE_NOP support to isiSendDataRequest()
 *
 * Revision 1.6  2003/12/04 23:31:57  dechavez
 * added IACP_TYPE_ALERT support to ReceiveDataRequestAcknowledgment
 *
 * Revision 1.5  2003/11/25 20:36:35  dechavez
 * added isiGetDatreqCompress()
 *
 * Revision 1.4  2003/11/21 20:03:23  dechavez
 * fixed typo in testing for isiCopyStreamListToDataRequest malloc failure,
 * removed some left over debugging printfs
 *
 * Revision 1.3  2003/11/19 23:50:46  dechavez
 * added return value for EINVAL isiAppendStreamReq call
 *
 * Revision 1.2  2003/11/03 23:53:45  dechavez
 * added isiSetDatreqPolicy(), isiSetDatreqFormat(), isiSetDatreqCompress(),
 * isiSendDataRequest(), isiClearDataRequest(), isiLocateStreamRequest(),
 * isiFreeDataRequest(), isiAllocSimpleDataRequest(), isiAllocDefaultDataRequest()
 * Data request's are now acknowledged with server expanded requests
 *
 * Revision 1.1  2003/10/16 15:38:52  dechavez
 * Initial release
 *
 */
