import seis
from seis.types	import *

def correctGain(traces, new_gain=1.E09):

    assert isinstance(traces, (list, tuple))

    for trc in traces:
        ss = trc.data.spacesaver()
        trc.data.savespace(1)
        trc.data *= new_gain/trc.gain
        trc.gain  = new_gain
        trc.data.savespace(ss)
#       trc.setGain(gain)


def sortChannels(traces, alphabet="LZQNET"):

    def cmp_cha (t1, t2):
        if   t1.cha == t2.cha:  return  0
        elif t1.cha  < t2.cha:  return  1
        else:			return -1

    traces.sort(cmp_cha)

    return traces


def dictFromChannels(traces):
    """
    creates a dictionary from a sequence of traces, the key being the
    last character of the channel identifier (e.g. 'Z' for 'BHZ')
    """
    d = {}
    for trc in traces:
        c = trc.cha[-1]
        if c in d:
            raise ValueError, "multiple channel ID's not allowed: %s" % c
        d[c] = trc
    return d

def cutPhase(traces, phases, offset, length, slowness_index=None):

    # XXX This slowness_index hack is rather messy

    import seis.ttt, seis.delta

    for trc in traces:
        delta, az1, az2 = seis.delta.delazi2(trc.evt, trc.sta)
        arrivals = seis.ttt.compute(delta, trc.evt.dep)
        a = arrivals.findFirstOf(phases)
        if not a: raise ValueError, "no arrivals found"
        atime = trc.evt.time + a.time
        if slowness_index:
            trc.extra[slowness_index] = a.slow
        trc.cut(offset, length, atime)

def rotate(trc1, trc2, phi):

    # the angle phi is measured from trc1 over trc2

    if not isinstance(trc1, Trace) or not isinstance(trc2, Trace):
        raise ValueError

    tc1, tc2 = trc1.data.typecode(), trc2.data.typecode()
    if tc1 in "fd" and tc1==tc2:
	import seis._numeric
	seis._numeric.rotate(trc1.data, trc2.data, phi)
    else:
        from Numeric import cos, sin, pi
        if (trc1.gain-trc2.gain)/(trc1.gain+trc2.gain) > 0.001:
            import sys
            sys.stderr.write("warning: different gains in 'rotate()'\n")
        cb, sb = cos(phi*pi/180.), sin(phi*pi/180.)
        x =  cb*trc1.data + sb*trc2.data
        y = -sb*trc1.data + cb*trc2.data
        if x.typecode() != tc1:
                trc1.data = x.astype(tc1)
        else:   trc1.data = x
        if y.typecode() != tc2:
                trc2.data = y.astype(tc2)
        else:   trc2.data = y


def VPVS(sigma):

    from Numeric import sqrt

    return sqrt((2.-2.*(sigma))/(1.-2.*(sigma)))


def _decomp(z, r, p, vs, sigma):

    # p is the slowness in sec/km

    if not isinstance(z, Trace) or not isinstance(r, Trace):
        raise ValueError

    from Numeric import sqrt
    vp = vs*VPVS(sigma)

    qa  = sqrt (1./(vp*vp)-p*p);
    qb  = sqrt (1./(vs*vs)-p*p);
    m11 = -(2*vs*vs*p*p-1.)/(vp*qa);
    m12 =   2.*p*vs*vs/vp;
    m21 =  -2.*p*vs;
    m22 =  (1.-2.*vs*vs*p*p)/(vs*qb);
    zz, rr = z*m11 + r*m12, z*m21 + r*m22
    if r.typecode() != rr.typecode():
            r[:] = rr.astype(r.typecode())
    else:   r[:] = rr[:]
    if z.typecode() != zz.typecode():
            z[:] = zz.astype(z.typecode())
    else:   z[:] = zz[:]


def decompose(traces, slowness, vs=3.5, sigma=0.25):
    """
    In-place wavefield decomposition
    The slowness p must be angular slowness in sec/deg
    """

    Z,N,E,R,L,Q,T = 'ZNERLQT' # readablity matters

    if len(traces) not in [2,3]:
        raise ValueError, "specify either 2 or 3 input records"

    correctGain(traces)
    t = dictFromChannels(traces)

    if N in t and E in t: # horizontal rotation
        from seis.delta import delazi2
        delta, baz, azi = delazi2 (t[N].sta, t[N].evt)
        rotate(t[N], t[E], baz+180.)
        t[R], t[T] = t[N], t[E]
        t[R].cha, t[T].cha = t[R].cha[:-1]+'R', t[T].cha[:-1]+'T'

    if vs > 0.001:
	if Z in t and R in t: # decomposition of Z/R into P/SV
	    decompose_zr(t[Z], t[R], slowness, vs, sigma)
	    t[L], t[Q] = t[Z], t[R]
	else: raise ValueError, "wrong components: %s" % \
                            ','.join([t.cha[-1] for t in traces])
	for c in L, Q, T:
	    t[c].cha = t[c].cha[:-1] + c
    else:
	for c in R, T:
	    t[c].cha = t[c].cha[:-1] + c

def decompose_zr(z, r, slowness, vs=3.5, sigma=0.25):
    """
    In-place wavefield decomposition of Z/R into P/SV
    """
    # slowness must be angular slowness in sec/deg
    if not isinstance(z, Trace) or not isinstance(r, Trace):
        raise ValueError

    import seis._numeric

    correctGain([z, r])

    cz, cr = z.cha, r.cha
    if cz[-1]=='Z' and cr[-1]=='R': # XXX or better skip this check?
        p = slowness*0.0089932 # convert angular slowness to sec/km
        fz, fr = z.data, r.data
        seis._numeric.decomp(fz, fr, p, max(vs, 0.01), sigma)
    else: raise ValueError, "wrong components [%c,%c]" % (cz[-1], cr[-1])

    z.cha, r.cha = cz[:-1]+'L', cr[:-1]+'Q'

def align(traces):
    # XXX Doesn't yet work if the data don't overlap. In that case,
    # it should throw an exception!!!
    if len(traces) == 0:
        return
    trc = traces[0]
    t0, t1 = trc.time, trc.endtime
    for trc in traces:
        _t0, _t1 = trc.time, trc.endtime
        if _t0 > t0: t0 = _t0
        if _t1 < t1: t1 = _t1
    for trc in traces:
#       trc.cut(trc.time-t0, t1-t0)
        trc.cut(0, t1-t0, t0)

def aligned(traces):
    """ test if the traces are properly time aligned """
    trc0 = traces[0]
    for trc in traces[1:]:
        if abs(trc.time - trc0.time) > 0.01:
            return False
        if trc.nsamp != trc0.nsamp:
            return False
        if trc.fsamp != trc0.fsamp:
            return False

    return True
def minmax(traces):
    # computes the minimum and maximum values of all traces
    # XXX to be revised
    min_, max_ = [], []
    for trc in traces:
	_min, _max = trc.minmax()
	min_.append(_min)
	max_.append(_max)
    return min(min_), max(max_)

