import seis._scipy_signal, Numeric as Num, FFT

# The next symbols *shall* be re-exported
from seis._numeric import nextPowerOf2, IIR_Filter, find_max
from seis._numeric import bw_low, bw_high, bw_band

__doc__ = """
This is a collection of "low-level" functions operating on
Numeric.array's and usually returning new Numeric.array's.
"""

def covar2(x, y):
    """
    Computes (co)variances of 2 arrays x, y of same size.
    The variances are returned as cxx, cyy, cxy
    """
    assert len(x)==len(y)

    sx = x - Num.average(x)
    sy = y - Num.average(y)

    # variances
    cxx = Num.average(sx*sx)
    cyy = Num.average(sy*sy)
    # co-variance
    cxy = Num.average(sx*sy)

    return cxx, cyy, cxy


def covar3(x, y, z):
    """
    Computes (co)variances of 3 arrays x, y, z of same size.
    The variances are returned as cxx, cyy, czz, cxy, cxz, cyz
    """
    assert len(x)==len(y)==len(z)
    assert x.typecode()==y.typecode()==z.typecode()

# XXX XXX XXX XXX XXX
#   sx = x - Num.average(x)
#   sy = y - Num.average(y)
#   sz = z - Num.average(z)
    sx = x
    sy = y
    sz = z

    if x.typecode() in ['f','d']:
        # variances
        cxx = Num.average(sx*sx)
        cyy = Num.average(sy*sy)
        czz = Num.average(sz*sz)
        # co-variances
        cxy = Num.average(sx*sy)
        cxz = Num.average(sx*sz)
        cyz = Num.average(sy*sz)

        cyx, czx, czy = cxy, cxz, cyz
        
    elif x.typecode() in ['F','D']:
        # for complex traces, compute [x,y,z]*adjoint([x,y,z])
        # (conjugate transpose)

        # variances
        cxx = Num.average(sx*Num.conjugate(sx))
        cyy = Num.average(sy*Num.conjugate(sy))
        czz = Num.average(sz*Num.conjugate(sz))
        # co-variances
        cxy = Num.average(sx*Num.conjugate(sy))
        cxz = Num.average(sx*Num.conjugate(sz))
        cyz = Num.average(sy*Num.conjugate(sz))

        cyx = Num.conjugate(cxy)
        czx = Num.conjugate(cxz)
        czy = Num.conjugate(cyz)
    else:
        raise TypeError

    return Num.array([ [cxx,cxy,cxz], [cyx,cyy,cyz], [czx,czy,czz] ])


def hilbert_py(x):
    """
    Returns the Hilbert transform of x (pure-Python version)
    """
    n    = len(x)
    nfft = nextPowerOf2(n)
    x = FFT.real_fft(x, nfft)
    x[1:-1] = x[1:-1]*1j
    x = FFT.inverse_real_fft(x, nfft)
    return x[:n]

def hilbert(x):
    """
    Returns the Hilbert transform of x
    """
    from seis._numeric import hilbert as _hilbert
    x = Num.array(x, copy=1)
    _hilbert(x)
    return x

def analytic(x):
    """
    Returns the so-called "analytic" signal, i.e.
    x + 1j*hilbert(x)
    """
    return x + 1j*hilbert(x)

def envelope(x):
    """
    Returns the envelope of x
    """
    return Num.absolute(analytic(x))

def instphas(x):
    """
    Compute the instantaneous phase of x
    UNFINISHED - DO NOT USE!!!
    """
    # NOT WELL TESTED
    y = hilbert(x)
    pha = Num.arctan2(x,y)
    # unwrap phase
    for i in xrange(1, len(pha)):
        if pha[i]>pha[i-1] and pha[i]-pha[i-1] > 4.:
            pha[i:] = pha[i:]-2*Num.pi
    return pha

def deriv(x):
    # NOT WELL TESTED
    dx = Num.array(x, copy=1)
    xx = x[1:]-x[:-1]
    dx[1:-1] = 0.5*(xx[1:]+xx[:-1])
    dx[ 0] = x[ 1] - x[ 0]
    dx[-1] = x[-1] - x[-2]
    return dx

def instfreq(x):
    """
    Compute the instantaneous frequency of the trace in place.
    UNFINISHED - DO NOT USE!!!
    """
    # NOT WELL TESTED - EXTREMELY SLOW!!!
            
    x = -deriv(instphas(x))
#   for i in xrange(1, len(x)-2):
#       if x[i]<-1 and x[i+1]<-1 and x[i-1]>0 and x[i+2]>0:
#           x[i]   = (2.*x[i-1] +    x[i+2])/3
#           x[i+1] = (   x[i-1] + 2.*x[i+2])/3
    return x

def instfreq2(x, fsamp=1.):
    x = analytic(x)
    x1 = x[:-1]
    x2 = x[1:]
    dphi = Num.arcsin(abs(x1.real*x2.imag-x1.imag*x2.real)/(abs(x1)*abs(x2)))
    return Num.concatenate([ [dphi[0]], dphi ])

def demean(x):
    """
    Returns the demeaned x
    """
    return x - Num.average(x)

def fit_line(f):
    """
    Determines the least squares best fit line of f
    """
    n = len(f)
    mx, mf, i = (n-1.)/2, Num.average(f), Num.arange(n)
    imx = i-mx
    mxf = Num.sum(imx*(f-mf))/(n-1.)
    sx2 = Num.sum(imx*imx)/(n-1.)
    bfx = mxf/sx2
    a, b = bfx, mf-bfx*mx
    return a, b 

def detrend(f, n=0):
    """
    Determines the least squares best fit line and removes it.
    """

    if n>0: a, b = (f[-1]-f[0])/len(f), f[0] # XXX ???
    else:   a, b = fit_line(f)

    i = Num.arange(len(f))
    return f - (a*i + b)

def taper(x, n1, n2=None):
    """
    Apply a cosine (Hanning) taper to x
    n1 and n2 are taper widths in samples. If n2 is omitted then
    n2=n1 is used. Tapering on either side can be turned off by
    setting the corresponding taper width to zero.
    """
    n  = len(x)
    x  = Num.array(x, copy=1)
    op = 0.5*(1.-Num.cos(Num.arange(int(n1))*(Num.pi/n1)))
    x[:int(n1)]  = x[:int(n1)] * op[:]
    if n2:  op = 0.5*(1.-Num.cos(Num.arange(int(n2))*(Num.pi/n2)))
    else:   n2 = n1
    x[-int(n2):] = x[-int(n2):]* op[::-1]
    return x

def __exp(x):
    return Num.exp(Num.maximum(x,-740.))

def __gauss_op(nfft, a):
    nf = nfft/2+1   # number of frequencies
    ff = (2*Num.pi/nfft)*Num.arange(0.,nf)
    qq = Num.sqrt(Num.pi)/a
    return qq*__exp(-(ff/(2*a))**2)

def gauss(x, a):
    """
    Apply a Gaussian low-pass filter

    The filter frequency a is relative to the sampling frequency.
    I.e. if the actual sampling frequency is 20 Hz, and the
    Gauss parameter shall be 4, a must be set to 0.2
    """
    n    = len(x)
    nfft = nextPowerOf2(n)
    ff   = FFT.real_fft(x, nfft)
    ff   = FFT.inverse_real_fft(ff*__gauss_op(nfft,a), nfft)
    return ff[:n]

#def find_max(x):
#    """
#    Returns the index and the amplitude of the maximum of x
#    """
#    imax, amax = 0, x[0]
#    for i,a in enumerate(x):
#        if a>amax:
#            imax, amax = i, a
#    return imax, amax

def measure_period(f, i0, dt=1.):
    """
    Measures the period of a more or less sinusoidal signal f about
    the sample with index i0. It does so by measuring the zero
    crossings of the original signal as well as its Hilbert transform.
    Optionally, the period is multiplied by the sampling interval dt of
    the series.
    An IndexError may be raised if those zero crossings are not
    within the time window.
    """
    def __period(f, i0):
        n = len(f)
        sign = Num.sign(f*Num.sign(f[i0]))
        i1 = i2 = i0
        while sign[i1]>=0: i1 = i1-1
        while sign[i2]>=0: i2 = i2+1
        while sign[i2]< 0: i2 = i2+1
        return i2-i1

    f = detrend(f)
    return 0.5*dt*( __period(f,i0) + __period(hilbert(f),i0) )


def IIR_ButterLowPass(fsamp, order, fc):

    from seis._numeric import IIR_Filter
    Wn = 2.*fc/fsamp
    b, a = seis._scipy_signal.butter(order, Wn, btype='lowpass')
    return IIR_Filter(a, b)

def IIR_ButterHighPass(fsamp, order, fc):

    from seis._numeric import IIR_Filter
    Wn = 2.*fc/fsamp
    b, a = seis._scipy_signal.butter(order, Wn, btype='highpass')
    return IIR_Filter(a, b)

def IIR_ButterBandPass(fsamp, order, fmin, fmax):

    from seis._numeric import IIR_Filter

    try:
        assert 0 <= fmin < fmax < fsamp and order > 0
        # some of the frequency relations my be relaxed by automatically
        # creating a lowpass if fmin==0 or highpass if fmax>=fsamp, but
        # let's for the time being consider this an error
    except:
        msg = "Inconsistent parameters to create IIR bandpass\n" \
              "Make sure that fmin < fmax < fsamp and order > 0"
        raise ValueError, msg

    Wn = (2.*fmin/fsamp, 2.*fmax/fsamp)
    b, a = seis._scipy_signal.butter(order, Wn, btype='bandpass')
    return IIR_Filter(a, b)


class FFT_Filter(object):
    
    def __init__(self, pole=None, zero=None, norm=1.):
        """ 
        Initialize filter from poles, zeros and normalization
        """
        if pole is None: pole = []
        if zero is None: zero = []
        self.pole = Num.array(pole)
        self.zero = Num.array(zero)
        self.norm = norm
        self.fsamp = None

    def __call__(self, f, fsamp=1.):
        self.apply(f, fsamp)

    def apply(self, f, fsamp=None):
        """
        Apply filter to trace
        """
        assert isinstance(f, Num.ArrayType)
        from FFT import real_fft, inverse_real_fft
        n, nfft = len(f), nextPowerOf2(len(f))
        ff = real_fft(f, nfft)
        try:    df = fsamp*1./nfft      # frequency increment
        except: df = self.fsamp*1./nfft # this may also fail, however
        nfreq = nfft/2+1                # number of frequencies
        fq = df*Num.arange(nfreq)       # array of frequencies
        ff = ff*self.response(fq)       # convolve with response
        ff = inverse_real_fft(ff, nfft)
        ff = ff[:n]                     # trim data to original length
        f[:] = ff.astype(f.typecode())  # cast data to original type

    def response(self, f):
        """
        Compute response at frequency/frequencies w
        w can be a either a number or an array.
        """
        iw   = 2j*Num.pi*f
        resp = self.norm
        # XXX
        for p in self.pole:
            resp = resp/(iw-p)
        for z in self.zero:
            resp = resp*(iw-z)
        return resp

    def grpdelay(self, f):
        """
        returns the group delay of the filter at the frequency f
        """
        rsp1 = self.response(f)
        rsp2 = self.response(f+0.001)
        pha1 = Num.arctan2(rsp1.imag, rsp1.real)
        pha2 = Num.arctan2(rsp2.imag, rsp2.real)
        return -(pha2-pha1)/(0.001*2*Num.pi)

    def phsdelay(self, f):
        """
        returns the phase delay of the filter at the frequency f
        """
        rsp = self.response(f)
        pha = Num.arctan2(rsp.imag, rsp.real)
        return -pha/(f*2*Num.pi)


def interp_lin (x, fx, default=None):
    n, i = len(fx), Num.floor (x)
    if i < 0 or i > n-2:
        if default==None:
#           msg = "interp_lin: value %f out of range [%f,%f]"
            msg = "interp_lin: value ot of range"
            raise ValueError, msg
        else:
            return default

    return fx[i] + (fx[i+1]-fx[i])*(x-i)


from seis._numeric import interp_lin2

def ___interp_lin2 (x, fx, fy, default=None):
    """ ***slow*** prototype """

    assert len(fx) == len(fy)
    assert len(fx) >= 2

    if x < fx[0]  or  x > fx[-1]:
        if default is None:
                raise ValueError, "x-value out of range"
        else:   return default

    from bisect import bisect
 
    i = bisect(fx, x) 
    x1, x2 = fx[i-1:i+1]
    y1, y2 = fy[i-1:i+1]
    y = (x-x1)/float(x2-x1)*(y2-y1) + y1

    return y

#fx = [1,3,8,9]
#fy = [1,-13,8,-9]

#for x in 1,1.5,2,3,4,8.5:
#    print interp_lin2 (x, fx, fy)

