#!/usr/bin/env python

from bisect import bisect, insort
from seis import Time, Event, delta, ttt
from seiscomp.autoloc_locsat import Origin
#from seis._numeric import pol_p
from Numeric import average, concatenate, sin, cos, exp, pi, sqrt, zeros, array, sum

max_rms_res = 2.5

class Beam(Event):

    look_back = 1800

    def __init__(self, diam=7., nmin=8, maxdist=180., **kwargs):
        Event.__init__(self, **kwargs)
        self.diam     = diam
        self.nmin     = nmin
        self.maxdist  = maxdist
        self.sortlist = array([], 'O')
        self.t0_list  = array([], 'd' ,savespace=1)
        self.w0_list  = []
        self.pck_ids  = array([], 'O')

    def setup(self, stations, staindex):
        self.stations = stations
        self.staindex = staindex
        nsta = len(self.stations)

        self.ttim = zeros(nsta, 'd', savespace=1)
        self.slow = zeros(nsta, 'd', savespace=1)
        self.dist = zeros(nsta, 'd', savespace=1)
        self.azi  = zeros(nsta, 'd', savespace=1)
        self.baz  = zeros(nsta, 'd', savespace=1)

        for i,sta in enumerate(self.stations):
            self.dist[i], self.azi[i], self.baz[i] = delta.delazi2(self, sta)
            d = self.dist[i]
            # XXX Need better criterion to select phases depending on
            # XXX distance. Or try to even match with various phases.
            if   d< 30: phases = [ "Pn", "Pb", "Pg", "P" ]
            elif d<120: phases = [ "P", "Pdiff" ]
            else:       phases = [ "PKiKP", "PKPdf" ]

            try:
                arr = ttt.compute(d, self.dep).findFirstOf(phases)
                assert arr is not None
                self.ttim[i] = arr.time
                self.slow[i] = arr.slow

#               inc[i] = pol_p(arr.slow, 3.0)  # XXX 3.0

            except AssertionError:
                self.ttim[i] = -1.

    def truncate(self, tmin):
        # truncate sortlist i.e. remove old arrivals befor tmin

        imin = bisect(self.t0_list, tmin.time1970)
        self.t0_list  = self.t0_list [imin:]
        self.sortlist = self.sortlist[imin:]
        self.pck_ids  = self.pck_ids [imin:]
        return

    def residuals(self, pckgroup, t_start=None):
        from MLab import median
        # compute residuals relative to the beam location
        t0 = zeros(len(pckgroup), 'd')
        sortlist = self.sortlist.tolist() # XXX somewhat ugly
        for i,pck in enumerate(pckgroup):
#           idx  = self.staindex[pck.sta]
            idx   = sortlist.index(pck)
            t0[i] = self.t0_list[idx]
        t0_ = median(t0)
        t0_res = t0 - t0_
        t0_rms = sqrt(average(t0_res**2))
        return t0_rms
# XXX

    def __find_group(self, xx, dt=60., dt0=4.):
        # try to find the largest group of picks in the time window
        # [t-dt,t+dt], to which pck belongs
        t0, wx, pck = xx
        # t0 must be a float time
        imin = bisect(self.t0_list, t0-dt)
        imax = bisect(self.t0_list, t0+dt)
        if imax-imin < self.nmin:
            return

        _lst = []   # pick list in time order
        _t0  = []
        for tx, pckx in zip(self.t0_list[imin:imax], self.sortlist[imin:imax]):
            if not pckx.used:
                continue
            _lst.append(pckx)
            _t0.append(tx)

        if pck not in _lst:
            # may happen if there are several picks from the same station
            return

        ipck = _lst.index(pck)
        lx = len(_lst)
        _cnt = [0.]*lx
        _flg = [0 ]*lx
        for i in xrange(0,lx):
            ti, wi, pcki = _t0[i], 1., _lst[i]
            idx  = self.staindex[pcki.sta]
            azi_i = self.azi[idx]
            slo_i = self.slow[idx]
            for k in xrange(i+1,lx):
                tk, wk, pckk = _t0[k], 1., _lst[k]
                idx = self.staindex[pckk.sta]
                azi_k = self.azi[idx]
                slo_k = self.slow[idx]
                azi_diff = abs(((azi_k-azi_i)+180.)%360-180.)
                # XXX 
                dtmax = 0.5*self.diam*(slo_i+slo_k) * azi_diff/90. + dt0
                if abs(ti-tk)<dtmax:
                    _cnt[i] += 1
                    _cnt[k] += 1
                    if i==ipck or k==ipck:
                        _flg[k] = 1
                        _flg[i] = 1
        if sum(_flg) < 6: # XXX self.nmin:
            return

        xmx = 0
        imx = 0
        tmx = 0
        pckgroup = []
        for i in xrange(0,lx):
            if not _flg[i]:
                continue
            ti, wi, pcki = Time(time1970=_t0[i]), 1., _lst[i]
            pckgroup.append(pcki)
            if _cnt[i] > xmx:
                xmx = _cnt[i]
                imx = i
                tmx = ti

        return tmx, pckgroup

    def feed(self, pck, add_only=False):

        # if add_only is True, the new pick is only added to the beam, but
        # no search for a pick group is performed.
        if pck.id in self.pck_ids:
            # but we may have to update the location
            return
        try:
            # Only use those picks for which amplitude and period
            # of the P wave have already been determined.
            peri_mb, ampl_mb = pck.peri_mb, pck.ampl_mb
            if peri_mb < 0.: return
        except:
            return

        try:
            # check if station is found in station data base
            i0 = self.staindex[pck.sta]
            tt = self.ttim[i0]
            if tt<0.: return
            p0 = self.slow[i0]
            t0 = pck.on.time1970 - tt
            distance = self.dist[i0]
        except (ValueError, KeyError):
            return

        w0 = 1. # hier Kriterien basierend auf Frequenz, BAZ etc.

        use_pick = 1
        # XXX this criterion is rather ad hoc and needs to be tested
        if distance > 35 and 0.0 < peri_mb < 0.6: use_pick = 0
        if distance > 65 and 0.0 < peri_mb < 0.7: use_pick = 0

        # determine the position at which to insert the pick into lists
        i0 = bisect(self.t0_list, t0)

        if not use_pick:
            return

        self.sortlist, self.t0_list, self.pck_ids = \
              concatenate([self.sortlist[:i0], [pck],    self.sortlist[i0:]]), \
              concatenate([self.t0_list [:i0], [t0],     self.t0_list [i0:]]), \
              concatenate([self.pck_ids [:i0], [pck.id], self.pck_ids [i0:]])

        if 114.5 < distance < 125: # XXX transition P/PKP can be critical
            return

        if add_only:
            return

        result = self.__find_group( (t0, w0, pck) )
        if not result:
            return

        org = Origin()
        org.lat, org.lon, org.dep = self.lat, self.lon, self.dep
        org.time, org.trgs = result

        rms = []
        for pck in org.trgs:
            i0 = self.staindex[pck.sta]
            tt = self.ttim[i0]
            dt = (org.time + self.ttim[i0] - pck.on)*5./self.slow[i0]
            rms.append(abs(dt)**2)
        org.rms_res = sqrt(average(rms))

        return org
