#!/usr/bin/env python

from __future__ import generators # for Python 2.2 compatibility

import  sys, os
import  Numeric, seis
from    seiscomp import mseed
from    seis import Time
from    seis.filters import detrend
from    seis.numeric import find_max
from    seiscomp.myconfig import MyConfig
from    seiscomp._detector import Trigger, BaseDetector as STA_LTA_Detector
from    seiscomp.postdetector import measure_ampl, measure_mb, \
        measure_ml, measure_polarization, ringbuf, measure_exact_time

verbose = 0
dt_max = 0.02

def msg(txt):
    seis.message(txt)

class TriggerConfig(MyConfig):

    # The following are items that must be present in
    # every section except in the 'default' section
    mandatory = ["net", "sta"]

    def readINI(self, filename):
        MyConfig.readINI(self, filename)
        default = self['default']

        for label in self:
            if label == "default":
                continue

            # for missing items, use the default
            for item in default:
                if item not in self[label]:
                    self[label][item] = default[item]

            # check for items that don't appear in the default
            for item in self[label]:
                if item not in default and item not in self.mandatory:
                    msg("[%s]: unmatched item '%s'" % (label, item))

            for item in self.mandatory:
                if item not in self[label]:
                    msg("[%s]: missing item   '%s'" % (label, item))
                    # XXX this could also be treated as a fatal error

            self[label]['label'] = label

        del self['default']

ntrg = 0


def _tail(f, forever=False):

    import time

    while True:
        line = f.readline()
        if line:
            yield line
        else:
            if forever:
                    time.sleep(1)
            else:   raise StopIteration

def TriggerInput2(infile=None, forever=False, timeout=0):

    import select

    if not infile:
        f = sys.stdin
    elif isinstance(infile, file):
        f = infile
    elif isinstance(infile, str):
        f = file(infile)
    else:
        raise TypeError, str(infile)

    flag = False
    while True:
        input, _output, _exc = select.select([ f ], [], [], timeout)
        if not input:
            yield None
            continue

        lines = []

        for line in _tail(f, forever=forever):
            if line and line[:5]=="BEGIN":
                flag = True
            if not flag:
                continue
            if not line or line[:3]=="END":
                break
            lines.append(line.strip())
   
        if not lines:
            break

        lines = '\n'.join(lines)

        trg = Trigger("")
        trg.read_new(lines)
        yield trg


def TriggerInput(infile=None, forever=False, timeout=None):

    if not infile:
        f = sys.stdin
    elif isinstance(infile, file):
        f = infile
    elif isinstance(infile, str):
        f = file(infile)
    else:
        raise TypeError, str(infile)

    flag = False
    while True:
        lines = []

        for line in _tail(f, forever=forever):
            if line and line[:5]=="BEGIN":
                flag = True
            if not flag:
                continue
            if not line or line[:3]=="END":
                break
            lines.append(line.strip())
   
        if not lines:
            break

        lines = '\n'.join(lines)

        trg = Trigger("")
        trg.read_new(lines)
        yield trg


def consecutive(rec1, rec2, time_tolerance=1.):
    # test if two records that arrive in order are actually "consecutive" in
    # the sense that the following properties must match:
    # * end time of the first with start time of the second, within a certain
    #   time_tolerance
    # * sampling frequency

    if not rec1 or not rec2:
        return False

    if rec1.stream_id != rec2.stream_id:
        return False

    dt = rec2.time-rec1.endtime
    if abs(dt) > 1.:
        msg("%-18s gap %7.1fs %s - %s " % (rec1.stream_id,
                    dt, str(rec1.endtime)[:-2], str(rec2.time)[:-2]))
        return False

    # sampling frequency
    if rec1.fsamp != rec2.fsamp:
        msg("%-18s fsamp changed from %s to %s Hz" \
            % ( rec1.stream_id, str(rec1.fsamp), str(rec2.fsamp)))
        return False

    return True

# The detector can be fed with Z component data only, or with 3
# component data. In the case of 3-component data, more parameters
# can be retrieved, but also 3 ring buffers must be maintained.

def detector_from_config(config):

    assert isinstance(config, dict)
    sta_len = float(self.config["sta_len"])
    lta_len = float(self.config["lta_len"])
    q_on    = float(self.config["trig_on"])
    q_off   = float(self.config["trig_off"])
    

def filter_from_config(config, fsamp):

    assert isinstance(config, dict)
    
    words = config["filter"].split()
    if words[0].lower() == "bandpass":
        from seis.numeric import IIR_ButterBandPass as BandPass
        filter_name, fmin, fmax, order = \
            words[0], float(words[1]), float(words[2]), int(words[3])
        return BandPass(fsamp, order, fmin, fmax)
    # XXX need to compute and save group delay
    else: raise ValueError, "'%s' is not an allowed filter type" % words[0]

    return filt


class Detector(object):
    """
    Simple STA/LTA event detector - doesn't handle multiplexed input

    Automatically recognizes time gaps in the data and in order to avoid
    numerical "event", reset()s itself after a time gap.
    """

    # TODO:
    #
    # * Make the detector more flexible and allow use of other
    #   characteristic functions than just STA/LTA.
    # * Improve configuration

    def __init__(self, config, wait_max=10., bad_records=None):
        # parameters needed from config:
        #   sta_len, lta_len
        #   q_on, q_off
        #   (fsamp is determined from the incoming data)

        assert isinstance(config, dict)
        self.config = config
        self.fsamp  = None
        self.tcorr  = -0.9 # FIXME - This must be computed - FIXME
        self.trgwin = None
        self.t_on   = None
        self.lastrec = None
        self.wait_max = wait_max
        self.bad_records = bad_records

    def __init_filter(self):

        if not self.fsamp:
            return

        self.filter = filter_from_config(self.config, self.fsamp)

    def __init_stalta(self):

        if not self.fsamp:
            return

        sta_len = float(self.config["sta_len"])
        lta_len = float(self.config["lta_len"])
        q_on    = float(self.config["trig_on"])
        q_off   = float(self.config["trig_off"])

        n_sta   = int(self.fsamp*sta_len)
        n_lta   = int(self.fsamp*lta_len)

        self.stalta = STA_LTA_Detector(n_sta, n_lta, q_on, q_off)

    def reset(self):

        if self.fsamp:
            self.__init_filter()
            self.__init_stalta()

    def feed(self, rec):
        """
        Feed a record to the detector
        In case of a detection, calls trigger_ON() and later trigger_OFF()
        """

        # Since the data may be unpacked "on the fly" and this may raise an
        # exception, the first thing we need to do here is try to unpack
        # the data. If this fails, we simply return.
        try:
            d = rec.data
        except:
            if self.bad_records:
                # save bad record to file so that we can check why
                f = file(self.bad_records, "w")
                f.write(rec.raw)
                f.close()
            return

        if not consecutive(self.lastrec, rec):
            self.fsamp = rec.fsamp # better retrieve from lastrec directly
            self.reset()
        self.lastrec = rec

        # save old trigger state
        if self.stalta.state > 0:
                old_state = 1
        else:   old_state = 0

	# compute characteristic function
        d = self.filter(d)  # band-pass filter the data
        s = self.stalta(d)  # apply STA/LTA filter to the data

        sta, lta, trig = s

        # if everywhere the trigger state is OFF we are done
        if not trig and old_state==0:
            return

        # ******** The following is executed ONLY if ********
        # ******** somewhere the trigger state is ON ********

        # add old trigger state to the beginning
        # XXX let self.stalta() do that
        trig = Numeric.concatenate([[old_state], trig])

        q = sta/lta         # compute the actual STA/LTA
        if self.t_on:       # update self.trgwin
            i0 = -len(self.trgwin)
            self.trgwin = Numeric.concatenate([self.trgwin, q])

        # number of samples to wait for a maximum
        if self.wait_max > 0.:
                wait_max_n = int(self.wait_max*rec.fsamp+0.5)
        else:   wait_max_n = 2000000000 # wait "indefinitely"

        # note that the 1st value of trig contains the old state
        for i,diff in enumerate(trig[1:]-trig[:-1]):
            if diff == 1:   # trigger state changes to ON
                t = rec.time + i/rec.fsamp + self.tcorr
                self.trgwin = q[i:]
                self.t_on = t
                i0 = i
                self.trigger_ON(t)

            if self.t_on and i-i0 == wait_max_n:
                trgwin = self.trgwin[:wait_max_n] # trgwin starts at -i0 !
                imax, amax = find_max(abs(trgwin))
                tmax = self.t_on + imax/rec.fsamp
                self.trigger_MAX(tmax, amax)

            if diff == -1:  # trigger state changes to OFF
                t = rec.time + i/rec.fsamp + self.tcorr
                if i-i0 < wait_max_n:
                    trgwin = self.trgwin[:i-i0]
                    imax, amax = find_max(abs(trgwin))
                    tmax = self.t_on + imax/rec.fsamp
                    self.trigger_MAX(tmax, amax)
                self.trigger_OFF(t)
                self.t_on = None

    def trigger_ON (self, t):
        """ Called when trigger changes its state to ON - to be overloaded """
        raise NotImplementedError, "trigger_ON not implemented"

    def trigger_OFF (self, t):
        """ Called when trigger changes its state to OFF - to be overloaded """
        raise NotImplementedError, "trigger_OFF not implemented"

    def trigger_MAX (self, t, a):
        """ Called when a maximum amplitude is found - to be overloaded
        The application must not assume that this is called immediately
        after the maximum is found; it may be called *any* time between
        two trigger_ON() and trigger_OFF() calls."""
        raise NotImplementedError, "trigger_MAX not implemented"


class MultiDetector(Detector):

    # tasks is a dictionary consisting of items with arbitrary key and value
    # that consist of a tuple containing:
    #   * a list of time delays
    #   * an action

#   tasks = {
#       "mb":([ 25. ], __det_amp_mb)
#       }

    def __init__(self, label, config, logger=sys.stderr, db=None):
        self.logger = logger
        self.db = db
        Detector.__init__(self, config)

        self.pending = {}

        for attr in [ "net", "sta", "cha", "loc", "label" ]:
            setattr(self, attr, config[attr])

    def feed(self, rec):

        id = rec.stream_id
        if id not in ringbuf:
            ringbuf[id] = mseed.RingBuffer(128, logfile=self.logger)
        # XXX The mseed.RingBuffer should take care of checking data continuity
        ringbuf[id].append(rec)

        if rec.cha[-1] != 'Z': # XXX this must be made configurable
            return

        Detector.feed(self, rec)

        now = rec.endtime
        self.__process_pending(now)

    def __det_amp_mb(self, key):

        try:
            result = measure_mb(self, key)
        except IndexError:
            # XXX There needs to be more detailed analysis of the
            # cause of the IndexError
            return

        if result:
            trg = self.pending[key]
            trg.ampl_mb, trg.time_mb, trg.peri_mb = result
            if trg.peri_mb > 3.:
                trg.peri_mb = -3.

    def __det_amp_ml(self, key):

        result = measure_ml(self, key)
        if result:
            trg = self.pending[key]
            trg.ampl_ml, trg.time_ml = result

    def __det_pol(self, key):

        result = measure_polarization(self, key)
        if result:
            trg = self.pending[key]
            trg.pol_p = result

    def __process_pending(self, now):
        """
        Process pending tasks for the triggers in self.pending, depending
        on the time lapse required to make sure enough data is avalable.
        """
        for key in self.pending.keys():

            trg = self.pending[key]
            dt  = now - trg.on

            if dt>25 and ( not hasattr(trg, "ampl_mb") or \
                ( hasattr(trg, "ampl_mb") and not trg.ampl_mb )):
                self.__det_amp_mb(key)
                self.__det_pol(key)
                measure_exact_time(self, key)
                
                try:
                    if trg.ampl_mb:
                        trg.timestamp = seis.time()
                        self.write_pick(trg)
                except AttributeError:
# XXX
#  File "/home/dcop/seiscomp/lib/python/seiscomp/detector.py", line 467, in  __process_pending
#    if trg.ampl_mb:
#  File "_detector.pyx", line 177, in _detector.Trigger.ampl_mb.__get__
#    AttributeError: ampl_mb
# XXX
                    pass

            if dt>150 and ( not hasattr(trg, "ampl_ml") or \
                ( hasattr(trg, "ampl_ml") and not trg.ampl_ml )):
                self.__det_amp_ml(key)

                try:
                    if trg.ampl_ml or dt > 1500:  # XXX dont keep old trg. w/o ML
                        trg.timestamp = seis.time()
                        self.write_pick(trg)
                        del self.pending[key]
                except AttributeError:
                    # XXX same problem as with ampl_mb can arise
                    pass

    def trigger_ON (self, t):
        id = self.__label(t)
        trg = Trigger(id, self.net, self.sta, self.loc, self.cha)
        trg.on = t
        trg.timestamp = seis.time()
        self.pending[id]  = trg
        self.currently_on = trg
        #   self.write_pick(trg)

    def trigger_OFF (self, t):
        trg = self.currently_on
        trg.off = t
        trg.timestamp = seis.time()
        self.currently_on = None
        self.write_pick(trg)

    def trigger_MAX (self, t, a):
        trg = self.currently_on
        trg.time_max = t
        trg.ampl_max = a
        trg.timestamp = seis.time()
        self.write_pick(trg)

    def write_pick(self, trg, f=sys.stdout):
        if str(trg.ampl_max) == "inf": return # XXX FIXME XXX

        f.write("%s\n" % str(trg))
        f.flush()

        if self.db: self.db.put_picks([trg])
                  
    def __label(self, t):
        tstr = "%02d%02d%02d.%02d%02d%05.2f" % t.asDate
        return "%s-%s" % (tstr, self.label)


def detector_matches_record(det, rec):

    if det.net      != rec.net:         return False
    if det.sta      != rec.sta:         return False
    if det.loc      != rec.loc:         return False
    if det.cha[:-1] != rec.cha[:-1]:    return False
    return True


class MyDetector(MultiDetector):
    pass
