#!/usr/bin/env python 

import sys, time, seis, seiscomp.detector
from   seiscomp._base_db \
import Config, get_config, _MyDB, paramstyle, xparamstyle
from   Numeric import average
from   seiscomp._datatypes import Origin as _Origin, Pick as _Pick
#from   seiscomp._datatypes import OriginError as _OriginError
from   seiscomp._datatypes import Association # as _Association

# XXX In the long term, it is planned to replace all occurences of seis.Time
# XXX by their datetime counterparts.

# This requires, that each data type to be store in the event_db be
# represented by a class defined here, and not somewhere else. 

# in pqsqlite the strings are all unicode's... thus they are converted
# using str() when read back in

# table names
table_event                 = "event"
table_origin                = "origin"
table_origin_confreg_locsat = "origin_confreg_locsat"
table_association           = "association"
table_pick                  = "pick"
table_station_info          = "sta_coord"
table_amplitude             = "amplitude"
table_station_magnitude     = "sta_mag"
table_network_magnitude     = "net_mag"

xcols = {
    table_origin:
        "time, time_usec, latitude, longitude, " \
        "depth, depth_type, time_error, latitude_error, " \
        "longitude_error, depth_error, event_id, agency, " \
        "location_method, ignored, id, location_type",
    table_association:
        "residual, used, origin_id, pick_id, phase",
    table_pick:
        "id, type, time, time_usec, net_code, sta_code, " \
        "ampl_max, time_max, ampl_mb, time_mb, peri_mb, " \
        "ampl_ml, time_ml, agency",
    table_station_info:
        "net_code, sta_code, latitude, longitude, elevation",
    table_amplitude:
        "pick_id, type, ampl, peri, time, time_usec",
    table_station_magnitude:
        "origin_id, net_code, sta_code, mag_value, mag_type"
}

xparam = {}
uparam = {}
for col in xcols:
    xparam[col] = ",".join(len(xcols[col].split(","))*[xparamstyle])
    uparam[col] = ",".join([x+"="+xparamstyle for x in xcols[col].split(",")])

schema = {

table_event:
"""
CREATE TABLE event (
    -- The event_id will become visible to the public.
    --
    -- This table seems a bit useless because it contains just one
    -- column, but it will probably be augmented in future.
    event_id        CHAR(20)        NOT NULL, -- e.g. "ev050102030405"
    PRIMARY KEY (event_id)
);
""",

table_origin:
"""
CREATE TABLE origin (
    -- This table contains origin information, e.g. time and
    -- coordinates associated with an event. An event may
    -- have several origins associated to it. The origin id is
    -- purely internal and will not be visible to the public.
    id              CHAR(20)        NOT NULL,   -- could be anything
    event_id        CHAR(20)        NOT NULL,   -- not yet used

--  location
    time            DATETIME        NOT NULL,
    time_usec       INTEGER         NOT NULL,   -- microseconds part of time
    latitude        DECIMAL(8,6)    NOT NULL,
    longitude       DECIMAL(9,6)    NOT NULL,
    depth           DECIMAL(7,3),
    depth_type      CHAR(2), -- default, automatic (with or without
                             -- depth phases), geophysicist etc.
                             -- NOT YET FULLY DEFINED!!!!
                             -- Currently only "A" is used
                             -- "F" would mean free depth
                             -- "G" would be a geophysicist-assigned depth
    location_type   CHAR(2)         NOT NULL,
                             -- A - automatic
                             -- M - manually revised
                             -- Q - (optional) questionable
    location_method VARCHAR(20),
--  Flinn-Engdahl region not saved, because it is now generated dynamically

--  location errors
    time_error      DECIMAL(7,3),
    latitude_error  DECIMAL(8,6),
    longitude_error DECIMAL(8,6),
    depth_error     DECIMAL(7,3),

    agency          CHAR(8)         NOT NULL,   -- e.g. "GFZ" or "BMG"
    time_modified   DATETIME,
    time_created    DATETIME,
    locked          TINYINT         DEFAULT 0,  -- if true, don't update origin
    ignored         TINYINT         DEFAULT 0,  -- if true, ignore origin
    PRIMARY KEY (id,location_type) -- maybe also "agency"?
); 
""",

table_origin_confreg_locsat:
"""
CREATE TABLE origin_confreg_locsat (
    -- Actually, the confidence regions are
    -- very dependent on the location program used.
    -- If LocSAT is used for the location, the confidence region
    -- shall be saved here.
    -- confreg_* refer to the 90% confidence region (LocSAT)
    id              CHAR(20)        NOT NULL,
    location_type   CHAR(2)         NOT NULL,
    confreg_level   DECIMAL(3,1), -- e.g. 90 %
    confreg_major   DECIMAL(5,1),
    confreg_minor   DECIMAL(5,1),
    confreg_strike  DECIMAL(4,1),
    confreg_time    DECIMAL(4,1),
    PRIMARY KEY (id)
);
""",

table_association:
"""
CREATE TABLE association (
    -- association between pick and origin/phase
    pick_id         CHAR(40)        NOT NULL,
    origin_id       CHAR(20)        NOT NULL,
    phase           CHAR(10)        NOT NULL, -- P, Pn, etc.
    residual        DECIMAL(5,2),             -- travel time residual in sec.
    used            BOOL        DEFAULT true, -- pick used to compute origin?
--  Future additions will include e.g.:
--  * azimuth
--  * slowness
--  * apparent incidence angle
    PRIMARY KEY (pick_id, origin_id, phase)
);
""",

table_amplitude:
"""
""",

table_station_magnitude:
"""
CREATE TABLE sta_mag (
    -- table containing individual magnitudes
    -- for each event, there can be one mag. value per mag. type per station
    -- NOTE that raw amplitudes/periods/times will be saved into a
    -- separate table which is not yet defined.
    net_code        CHAR(2)         NOT NULL,
    sta_code        CHAR(5)         NOT NULL,
    origin_id       CHAR(20)        NOT NULL,
    mag_type        BINARY CHAR(4)  NOT NULL, -- mag. type e.g. "mb" or "Ms"
    mag_value       DECIMAL(4,2)    NOT NULL, -- single mag. measurement
    PRIMARY KEY (origin_id, net_code, sta_code, mag_type)
);
""",

table_network_magnitude:
"""
CREATE TABLE net_mag (
    origin_id       CHAR(20)        NOT NULL,
    mag_type        BINARY CHAR(4)  NOT NULL, -- type of magnitude
    mag_num         INT(4)          NOT NULL, -- number of mag. measurements
    mag_avg         DECIMAL(4,2)    NOT NULL, -- average magnitude
--  mag_std         DECIMAL(4,2)    NOT NULL, -- standard deviation
    mag_med         DECIMAL(4,2)    NOT NULL, -- median magnitude
    PRIMARY KEY (origin_id, mag_type)
);

""",

table_pick:
"""
CREATE TABLE pick (
    id              CHAR(40)        NOT NULL,
    net_code        CHAR(2)         NOT NULL,
    sta_code        CHAR(5)         NOT NULL,
    time            DATETIME        NOT NULL,
    time_usec       INTEGER         NOT NULL,
--  date            DATE            NOT NULL,
--  time            DECIMAL(9,3)    NOT NULL,

    -- this amplitude is a property of the pick itself and is
    -- therefore not stored in a separate amplitude table
    ampl_max        DECIMAL(10,2),
    time_max        DECIMAL(5,2),

    -- It's not clear where to best store these amplitudes.
    -- Especially mb is determined right after the pick when
    -- there may not yet be any associated event
    ampl_mb         DECIMAL(10,2),
    time_mb         DECIMAL(5,2),
    peri_mb         DECIMAL(5,2),
    ampl_ml         DECIMAL(10,2),
    time_ml         DECIMAL(5,2),
    ampl_mb_bb      DECIMAL(10,2),  -- for broadband mB
    time_mb_bb      DECIMAL(5,2),   -- for broadband mB

    -- before the association, we usually cannot be sure what phase
    -- was picked. But especially for manual picks, the analyst can
    -- often tell from experience, so this information is entered
    -- here if available. NOT YET USED!
    phase_hint      CHAR(10)    DEFAULT "P",

    -- type is 'a' for automatic, 'm' for manual. 2nd char reserved for future
    type            CHAR(2)         NOT NULL DEFAULT "a",

    -- onset properties are not yet determined but will be in future
    -- The 1st character is either 'e' or 'i' (emergent/impulsive)
    -- The 2nd character is either 'c' or 'd' (compressional/dilatational)
    onset           CHAR(2),        -- 'e' or 'i' + 'c' or 'd'

    -- agency code, e.g. "GFZ" or "BMG" - 5 characters should be enough
    agency          CHAR(5)         NOT NULL,

    -- analyst initials - not yet used but will become important in future
    author          CHAR(3)         NOT NULL DEFAULT "xyz",

    PRIMARY KEY (id,type,agency,author)
);
""",

table_station_info:
"""
-- The station coordinates table is currently needed in order not to
-- require the user to install the whole SeisComP inventory database,
-- while not having to rely on separate files either.
-- This is an ad-hoc approach, however, which may be replaced by
-- something better in future.

CREATE TABLE sta_coord (
    net_code        CHAR(2)         NOT NULL,
    sta_code        CHAR(5)         NOT NULL,
    latitude        DECIMAL(8,6)    NOT NULL,
    longitude       DECIMAL(9,6)    NOT NULL,
    elevation       DECIMAL(7,3)    NOT NULL,
    PRIMARY KEY (net_code, sta_code)
);
"""
}


# FIXME agency to be made configurable
agency = "GFZ"


def errorexit(msg):
    sys.stderr.write("%s\n" % msg)
    sys.exit(1)    


def _dbtime(t):
    assert isinstance(t, seis.Time)
    return t.strftime("%Y-%m-%d %H:%M:%S"), int((t.rawsec % 1.)*1000000)


def _isotime(t):
    """
    Converts a time object with strftime() method (e.g. seis.Time or
    datetime.datetime) or valid time string to a ISO 8601 time string
    """

    if isinstance(t, str):
        t = seis.Time(t)

    try:
        return t.strftime("%Y-%m-%d %H:%M:%S") + (".%06d" % t.microsecond)
    except:
        raise TypeError, "isotype(): cannot convert '%s'" % type(t)

def _twin2sql(twin):
    """
    Converts a time window to an SQL WHERE clause, suitable for SeisComP DB
    like splitting of a time into DATE and FLOAT.
    """
    if not twin:
        return ""

    add_expr = "DATE_ADD(time, INTERVAL time_usec MICROSECOND)"

    tmin,tmax = twin
    if tmin and tmax:
        return "(%s BETWEEN \"%s\" AND \"%s\")" \
            % (add_expr, _isotime(tmin), _isotime(tmax))
    # Now tmin and/or tmax is None
    if tmin:
        return "%s >= \"%s\"" % (add_expr, _isotime(tmin))
    if tmax:
        return "%s <= \"%s\"" % (add_expr, _isotime(tmax))
    # Now tmin *and* tmax are None
    return ""

def _xwin2sql(xname, xwin):
    # generic window of field xname
    if not xwin:
        return ""
    xmin,xmax = xwin
    if xmin and xmax:
        return "(%s BETWEEN %f AND %f)" % (xname, xmin, xmax)
    # Now xmin and/or xmax is None
    if xmin:
        return "%s >= %f" % (xname, xmin)
    if latmax:
        return "%s <= %f" % (xname, xmax)
    # Now xmin *and* xmax are None
    return ""


def _lonwin2sql(lonwin):
    # This is special because longitude is cyclic and we have to deal with
    # cases where lonmin>lonmax, i.e. where the selection area crosses the
    # date line (180 deg E/W).
    if not lonwin:
        return ""
    lonmin,lonmax = lonwin
    if lonmin and lonmax:
        if lonmax-lonmin >= 360:
            lonmin,lonmax = -180, 180
        else:
            if not -180 <= lonmin <= 180:lonmin = (lonmin+180)%360-180
            if not -180 <= lonmax <= 180:lonmax = (lonmax+180)%360-180
        if lonmax < lonmin:
            return "(longitude BETWEEN %f AND 180 OR " \
                    "longitude BETWEEN -180 AND %f)" % (lonmin, lonmax)
        else:
            return "(longitude BETWEEN %f AND %f)" % (lonmin, lonmax)
    # lonmin and/or lonmax is None
    if lonmin:
        return "longitude >= %f" % lonmin
    if lonmax:
        return "longitude <= %f" % lonmax
    # Both lonmin and lonmax are now None
    return ""


def _latwin2sql(latwin):
    return _xwin2sql("latitude", latwin)


def _depwin2sql(depwin):
    return _xwin2sql("depth", depwin)


class Event(object):
    """UNFINISHED"""
    def get_origin_ids(self):
        cols = "id"
        sql  = "SELECT origin_id FROM origin" \
               "WHERE event_id=\"%s\"" %  self.id
        self.cur.execute(sql)
        origin_ids = []
        for origin_id in self.cur.fetchall():
            origin_ids.extend(origin_id)
            
        return origin_ids

    def get_origins(self):
        # get all associated origins XXX
        cols = "id, phase"
        sql  = "SELECT pick_id, phase FROM association " \
               "WHERE event_id=\"%s\"" %  self.id
        self.cur.execute(sql)
        picks = []
        for pick_id, phase in self.cur.fetchall():
            picks.extend(db.get_picks(id=pick_id))
            
        return picks


class Station(seis.Station): pass


class Amplitude(object):

    def __init__():
        pass 


class Pick(_Pick):

    def save(self):
        pass

    def load(self):
        pass


class Origin(_Origin):

    def __get_latitude(self):       return self.lat
    def __set_latitude(self, lat):  self.lat = lat
    def __get_longitude(self):      return self.lon
    def __set_longitude(self, lon): self.lon = lon
    def __get_depth(self):          return self.dep
    def __set_depth(self, dep):     self.dep = dep

    latitude  = property(__get_latitude,  __set_latitude)
    longitude = property(__get_longitude, __set_longitude)
    depth     = property(__get_depth,     __set_depth)

    def __init__(self,  id="",
                        time=None, latitude=None, longitude=None, depth=None,
                        depth_type = "A",
                        time_error = None, latitude_error = None,
                        longitude_error = None, depth_error = None,
                        agency="GFZ",
                        location_type="A", location_method = "locsat",
                        db=None):

        _Origin.__init__(self, id, time, latitude, longitude, depth)

        self.event_id           = "FIXME"
        self.depth_type         = depth_type
        self.location_type      = location_type
        self.location_method    = location_method
        self.agency             = agency
        self.db                 = db
        self.magnitudes         = None
        self.ignored            = 0

        self.time_error      = time_error
        self.latitude_error  = latitude_error
        self.longitude_error = longitude_error
        self.depth_error     = depth_error

    def get_associated_picks(self, phase=None):
        picks = []
        associations = self.get_associations()
        for a in associations:
            p = self.db.pick(a.pick_id)
            picks.append( (p, a.phase) )
        return picks

    def get_associations(self, pick_id=None, phase=None, used=None):

        return self.db.get_associations(origin_id=self.id, pick_id=pick_id,
                                        phase=phase, used=used)

    def available_magnitude_types(self):

        mag_types = []
        for m in self.magnitudes:
            if m.mag_type not in mag_types:
                mag_types.append(m.mag_type)

        return mag_types

    def average_magnitude(self, mag_type):

        mag = [ m.mag_value for m in self.magnitudes if m.mag_type==mag_type ]

        if not mag:
            return 0, 0.

        return len(mag), average(mag)

    def summary_magnitude(self, mag_types = None):
        # mag_types may be a list of magnitudes to be
        # used for computing the summary magnitude
        mag = []
        for m in self.magnitudes:
            if mag_types and m.mag_type not in mag_types:
                continue
            mag.append(m.mag_value)

        return len(mag), average(mag)

    def load_magnitudes(self):

        self.magnitudes = self.db.get_sta_magnitudes(origin_id=self.id)

    def __in_db(self):
        sql = "select id from origin where id=\"%s\"" % self.id
        cur = self.db.cursor()
        cur.execute(sql)
        fetch_result = cur.fetchall()
        if len(fetch_result):
            return True
        return False

    def save(self):
        table = table_origin
        if self.__in_db():
            sql  = "UPDATE %s " % table
            sql += "SET time=%s, time_usec=%s, latitude=%s, longitude=%s, " \
                   "depth=%s, depth_type=%s, " \
                   "time_error=%s, latitude_error=%s, longitude_error=%s, " \
                   "depth_error=%s, event_id=%s, agency=%s, " \
                   "location_method=%s, ignored=%s " \
                   "WHERE id=%s AND location_type=%s"
        else:
            sql  = "INSERT INTO %s (%s) " % (table, xcols[table])
            sql += "VALUES (%s)" % xparam[table]
        t,t_usec = _dbtime(self.time)
        val = (t, t_usec, self.lat, self.lon, self.dep, \
               self.depth_type, self.time_error, self.latitude_error, \
               self.longitude_error, self.depth_error, \
               self.event_id, self.agency, \
               self.location_method, self.ignored, \
               self.id, self.location_type)
        cur = self.db.cursor()
        cur.execute(sql, val)

    def load(self):
        table = table_origin
        sql   = "SELECT %s FROM %s" % (xcols[table], table)
        where_sql = [ "id=\"%s\"" % self.id,
                      "location_type=\"%s\"" % self.location_type ]
        sql += " WHERE %s" % (" AND ".join(where_sql))

        result = []
        cur = self.db.cursor()
        cur.execute(sql)
        fetch_result = cur.fetchall()
        if len(fetch_result) != 1:
            raise OperationalError
        t, t_usec, self.lat, self.lon, self.dep, self.depth_type, \
            self.time_error, self.latitude_error, \
            self.longitude_error, self.depth_error, \
            self.event_id, self.agency, self.location_method, self.ignored, \
            id,loc_typ = fetch_result[0]
        self.time = seis.Time(str(t)) + t_usec/1000000.


class StationMagnitude(object):

    def __init__ (self, origin_id, net, sta, mag_type, mag_value):
        self.origin_id = origin_id
        self.net, self.sta = net, sta
        self.mag_type, self.mag_value = mag_type, mag_value


class AverageMagnitude(object): pass

class EventDB(_MyDB):

    def origin(self, origin_id, loctype="A"):
        org = self.get_origins(origin_id, loctype=loctype)
        if not org:
            return None
        return org[0]

    def get_origins(self, id=None, twin=None,
                    latwin=None, lonwin=None, depwin=None,
                    deptype=None, loctype=None, agency=None,
                    order=False, limit=None):

        """Note that the resolution of time window is 1 second."""

        table = table_origin
        sql   = "SELECT id, location_type FROM %s" % table
        if id:
            where_sql = [ "id=\"%s\"" % id ]
        else:
            where_sql = [ "ignored=0" ]
            if agency is not None:  where_sql.append("agency=\"%s\"" % agency)
            if twin   is not None:  where_sql.append(_twin2sql(twin))
            if latwin is not None:  where_sql.append(_latwin2sql(latwin))
            if lonwin is not None:  where_sql.append(_lonwin2sql(lonwin))
            if depwin is not None:  where_sql.append(_depwin2sql(depwin))

        if loctype:
            where_sql.append("location_type=\"%s\"" % loctype)

        if where_sql:
            where_sql = [ x for x in where_sql if x ]
            sql += " WHERE %s" % (" AND ".join(where_sql))
        if order:
            sql += " ORDER BY time DESC, time_usec DESC"
        if limit is not None:
            sql += " LIMIT %d" % limit

        result = []
        cur = self.cursor()
        cur.execute(sql)
        for id, loc_typ in cur.fetchall():
            org = Origin(id=str(id), location_type=str(loc_typ), db=self)
            org.load()
            result.append(org)

        return result

    def put_origins(self, origins):

        for org in origins:
            org.db = self
            org.save()

    def ignore_origin(self, origin):
        table = table_origin
        sql   = "UPDATE TABLE %s " % (table,)
        sql  += "SET ignored=1 WHERE id=\"%s\"" % origin.id
        cur = self.cursor()
        cur.execute(sql)
        origin.ignored = 1

    def pick(self, pick_id):
        p = self.get_picks(pick_id)
        if p: return p[0]
        return None

    def get_picks(self, id=None, twin=None, net=None, sta=None, agency=None,
                    order=False, limit=None):

        """Note that the resolution of time window is 1 second."""

        table = table_pick
        sql   = "SELECT %s FROM %s" % (xcols[table], table)
        if id:
            where_sql = ["id=\"%s\"" % id]
        else:
            where_sql = []
            if net:     where_sql.append("net_code=\"%s\"" % net)
            if sta:     where_sql.append("sta_code=\"%s\"" % sta)
            if agency:  where_sql.append("agency=\"%s\"" % agency)
            if twin:    where_sql.append(_twin2sql(twin))
        if where_sql:
            where_sql = [ x for x in where_sql if x ]
            sql += " WHERE %s" % (" AND ".join(where_sql))
        if order:
            sql += " ORDER BY time DESC, time_usec DESC"
        if limit is not None:
            sql += " LIMIT %d" % limit

        cur = self.cursor()
        cur.execute(sql)

        result = []
        for id, ptype, t, t_usec, net, sta, ampl_max, time_max, ampl_mb, \
            time_mb, peri_mb, ampl_ml, time_ml, agency \
        in cur.fetchall():
            p = Pick(id=str(id), net=str(net), sta=str(sta), loc="")
            p.time = seis.Time(str(t)) + t_usec/1000000.
            p.agency = str(agency)
# XXX       p.time, p.agency = time, agency
#           print "ampl_max=",ampl_max
            if ampl_max is not None:    # expect time_max also not None
                p.ampl_max,p.time_max = ampl_max,p.time+time_max
            if ampl_mb is not None:     # expect time_mb,peri_mb also not None
                p.ampl_mb,p.time_mb,p.peri_mb = ampl_mb,p.time+time_mb,peri_mb
            if ampl_ml is not None:     # expect time_ml also not None
                p.ampl_ml,p.time_ml = ampl_ml,p.time+time_ml
            result.append(p)

        return result

    def put_picks(self, picks):

        table = table_pick
        sql   = "REPLACE INTO %s (%s) " % (table, xcols[table])
        sql  += "VALUES (%s)" % xparam[table]

        for p in picks:
            try:                    agency =    p.agency
            except AttributeError:  agency = self.agency

            try:                    pick_type = p.pick_type
            except AttributeError:  pick_type = "A"

            t, t_usec = _dbtime(p.time)

            try:    ampl_max, time_max = p.ampl_max, p.time_max-p.time
            except: ampl_max, time_max = None, None

            try:    ampl_mb,time_mb,peri_mb = \
                        p.ampl_mb,p.time_mb-p.time,p.peri_mb
            except: ampl_mb,time_mb,peri_mb = None, None, None

            try:    ampl_ml, time_ml = p.ampl_ml, p.time_ml-p.time
            except: ampl_ml, time_ml = None, None

            val = ( p.id, pick_type, t, t_usec, p.net, p.sta,
                    ampl_max, time_max, ampl_mb, time_mb, peri_mb,
                    ampl_ml, time_ml, agency)
            cur = self.cursor()
            cur.execute(sql, val)

    def get_sta_magnitudes(self, origin_id=None, net=None, sta=None,
                                 mag_type=None):

        table = table_station_magnitude
        sql   = "SELECT %s FROM %s" % (xcols[table], table)

        where_sql = []
        if origin_id:   where_sql.append("origin_id=\"%s\"" % origin_id)
        if net:         where_sql.append("net_code=\"%s\"" % net)
        if sta:         where_sql.append("sta_code=\"%s\"" % sta)
        if mag_type:    where_sql.append("mag_type=\"%s\"" % mag_type)
        if where_sql:
            where_sql = [ x for x in where_sql if x ]
            sql += " WHERE %s" % (" AND ".join(where_sql))

        cur = self.cursor()
        cur.execute(sql)
        result = []
        for origin_id, net, sta, mag_value, mag_type in cur.fetchall():
            m = StationMagnitude(str(origin_id), str(net), str(sta),
                                 str(mag_type), mag_value)
            result.append(m)

        return result

    def put_sta_magnitudes(self, magnitudes):

        table = table_station_magnitude
        if len(magnitudes)==0: return

        cur = self.cursor()

        origin_id = magnitudes[0].origin_id
        # XXX Currently all existing station magnitudes are deleted for
        # XXX a given origin_id. A better approach is needed here.
        if paramstyle == "format":
            sql = "DELETE FROM " + table + " WHERE origin_id=%s"
        elif paramstyle == "qmark":
            sql = "DELETE FROM " + table + " WHERE origin_id=?"
        cur.execute(sql, (origin_id,))

        sql   = "REPLACE INTO %s (%s) " % (table, xcols[table])
        sql  += "VALUES (%s)" % xparam[table]

        for m in magnitudes:
#           try:                    agency =    p.agency
#           except AttributeError:  agency = self.agency

            val = m.origin_id, m.net, m.sta, m.mag_value, m.mag_type
            cur.execute(sql, val)

    def put_amplitudes(self, amplitudes):

        table = table_amplitude
        sql   = "REPLACE INTO %s (%s) " % (table, xcols[table])
        sql  += "VALUES (%s)" % xparam[table]

        cur = self.cursor()
        for a in amplitudes:
            t, t_usec = _dbtime(p.time)

            val = (a.pick_id, a.type, a.ampl, a.peri, t, t_usec)
            cur.execute(sql, val)

    def get_amplitudes(self, amplitudes):

        table = table_amplitude
        sql   = "SELECT %s FROM %s" % (xcols[table], table)

# XXX FIXME XXX
# NOT YET COMPLETED
        cur = self.cursor()
        for a in amplitudes:
            t, t_usec = _dbtime(p.time)

            val = ( a.pick_id, a.type, a.ampl, a.peri, t, t_usec )
            cur.execute(sql, val)

    def get_associations(self, origin_id=None, pick_id=None,
                               phase=None, used=None):

        table = table_association
        sql   = "SELECT %s FROM %s" % (xcols[table], table)

        where_sql = []
        if origin_id is not None:
            where_sql.append("origin_id=\"%s\"" % origin_id)
        if pick_id   is not None:
            where_sql.append("pick_id=\"%s\"" % pick_id)
        if phase     is not None:
            where_sql.append("phase=\"%s\"" % phase)
        if used      is not None:
            if used:  used = 1
            else:     used = 0
            where_sql.append("used=\"%s\"" % used)
        if where_sql:
            where_sql = [ x for x in where_sql if x ]
            sql += " WHERE %s" % (" AND ".join(where_sql))

        result = []
        cur = self.cursor()
        cur.execute(sql)
        for residual, used, origin_id, pick_id, phase in cur.fetchall():
            a = Association(origin_id=str(origin_id), pick_id=str(pick_id),
                            phase=str(phase), residual=residual, used=used,
                            db=self)
            result.append(a)

        return result

    def put_associations(self, associations):

        table = table_association
        # XXX We currently delete all existing associations for a given
        # XXX origin_id. We need a different approach here.
        cur = self.cursor()
        origin_id = associations[0].origin_id

        if paramstyle == "format":
            sql = "DELETE FROM " + table + " WHERE origin_id=%s"
        elif paramstyle == "qmark":
            sql = "DELETE FROM " + table + " WHERE origin_id=?"
        cur.execute(sql, (origin_id,))

        sql   = "REPLACE INTO %s (%s) " % (table, xcols[table])
        sql  += "VALUES (%s)" % xparam[table]

        for a in associations:
            val = ( a.residual, a.used, a.origin_id, a.pick_id, a.phase )
            cur.execute(sql, val)

    def get_sta_coord(self, network=None, code=None):

        table = table_station_info
        sql   = "SELECT %s FROM %s" % (xcols[table], table)
        where_sql  = []
        if network: where_sql.append("net_code=\"%s\"" % network)
        if code:    where_sql.append("sta_code=\"%s\"" % code)
        if where_sql:
            where_sql = [ x for x in where_sql if x ]
            sql += " WHERE %s" % (" AND ".join(where_sql))

        result = []
        cur = self.cursor()
        cur.execute(sql)
        for net, code, lat, lon, alt in cur.fetchall():
            sta = Station(net=str(net), code=str(code),
                          lat=lat, lon=lon, alt=alt)
#           sta.desc = description
            result.append(sta)

        return result

    def put_sta_coord(self, stations):

        table = table_station_info
        sql   = "REPLACE INTO %s (%s) " % (table, xcols[table])
        sql  += "VALUES (%s)" % xparam[table]

        cur = self.cursor()
        for s in stations:
            val = ( s.net, s.code, s.lat, s.lon, s.alt )
            cur.execute(sql, val)
